#pragma once

#include <stdint.h>
#include "board.hpp"
#include "measurer.hpp"
#include "voltage.hpp"
#include "configuration.hpp"
#include "player.hpp"
#include "reader.hpp"
#include "button.hpp"


namespace Voltmeter
{
    class Application
    {
        public:

            constexpr static int16_t const LongPressTimeMs = 1000;
            constexpr static int16_t const BacklightTimeMs = 3000;

            constexpr static uint8_t const VolumeLevel0 = 15;
            constexpr static uint8_t const VolumeLevel1 = 8;
            constexpr static uint8_t const VolumeLevel2 = 6;
            constexpr static uint8_t const VolumeLevel3 = 4;

            constexpr static Board::WatchDog::Timeout MainLoopTimeout = Board::WatchDog::Timeout::_60ms;
            constexpr static Board::WatchDog::Timeout SdInitTimeout = Board::WatchDog::Timeout::_500ms;
            constexpr static Board::WatchDog::Timeout ConfigWriteTimeout = Board::WatchDog::Timeout::_500ms;

            Application();

            void Main();
            void AdcIntHandler();
            void Timer1OverflowIntHandler();
            void Timer2OverflowIntHandler();

        private:

            Configuration configuration;
            Measurer measurer;
            Voltage dcVoltage;
            Voltage acVoltage;
            Player player;
            Reader reader;

            int32_t timeMs;

            Button OnOffButton;
            Button RangeButton;
            Button TalkButton;

            uint8_t volumeDivider;

            bool isBacklightOn;
            int32_t backlightOnTime;

            void ErrorP(const char * message, bool isFatal);
            int32_t GetTimeMs();

            void Measure();
            void GetInput();
            void UpdateDisplay();

            uint8_t VolumeToVolumeDivider(uint8_t volume);
            void ChangeVolume();

            void Calibrate();
            void CalibrateRange(Range const range);

            void PowerOff();
    };
}