#pragma once

#include <stdint.h>
#include "utils.hpp"


namespace Devices::I2c
{
    template <class I2cImplT>
    class Memory
    {
        public:

            static bool Init()
            {
                return I2cImplT::Init();
            }


            static bool Write(uint8_t deviceAddress, uint8_t startAddress, uint8_t length, void* buffer)
            {
                bool result = true;

                result = result && I2cImplT::Start(deviceAddress, I2c::Direction::Transmit);
                result = result && I2cImplT::Transmit(startAddress);
                result = result && I2cImplT::Transmit(length, buffer);
                result = result && I2cImplT::Stop();
                
                return result;
            }


            static bool Write(uint8_t deviceAddress, uint8_t address, uint8_t value)
            {
                return Write(deviceAddress, address, 1, &value);
            }


            static bool Read(uint8_t deviceAddress, uint8_t startAddress, uint8_t length, void* buffer)
            {
                bool result = true;
                
                result = result && I2cImplT::Start(deviceAddress, I2c::Direction::Transmit);
                result = result && I2cImplT::Transmit(startAddress);
                result = result && I2cImplT::Stop();

                result = result && I2cImplT::Start(deviceAddress, I2c::Direction::Receive);
                result = result && I2cImplT::ReceiveNack(length, buffer);
                result = result && I2cImplT::Stop();
                
                return result;
            }


            static bool Read(uint8_t deviceAddress, uint8_t address, uint8_t &result)
            {
                return Read(deviceAddress, address, 1, &result);
            }


            static bool ReadModifyWrite(uint8_t deviceAddress, uint8_t address, uint8_t const SetBits, uint8_t const ClearBits)
            {
                uint8_t reg;

                if (!Read(deviceAddress, address, reg))
                {
                    return false;
                }

                reg = (reg & ~ClearBits) | SetBits;

                return Write(deviceAddress, address, reg);
            }


            static bool SetBit(uint8_t deviceAddress, uint8_t address, uint8_t const bit, bool condition)
            {
                uint8_t set = (1 << bit);
                uint8_t clear = 0;

                if (!condition)
                {
                    clear = set;
                    set = 0;
                }

                return ReadModifyWrite(deviceAddress, address, set, clear);
            }
    };
}