#pragma once

#include <stdint.h>


namespace Devices::Mcu
{
    template <int vectorNum, class pin, uint8_t ISCn1, uint8_t ISCn0, uint8_t INTn, uint8_t INTFn>
    struct ExternalInterrupt
    {
        using Pin = pin;
        constexpr static int VectorNum = vectorNum;

        enum class SenseControl
        {
            LowLevel = 0x00,
            Change = 0x01,
            FallingEdge = 0x02,
            RisingEdge = 0x03
        };


        static void Enable(SenseControl senseControl)
        {
            switch (senseControl)
            {
                case SenseControl::LowLevel:
                    EICRA &= ~_BV(ISCn1) & ~_BV(ISCn0);
                    break;

                case SenseControl::Change:
                    EICRA &= ~_BV(ISCn1);
                    EICRA |= _BV(ISCn0);
                    break;

                case SenseControl::FallingEdge:
                    EICRA |= _BV(ISCn1);
                    EICRA &= ~_BV(ISCn0);
                    break;

                case SenseControl::RisingEdge:
                    EICRA |= _BV(ISCn1) | _BV(ISCn0);
                    break;
            }

            EIMSK |= _BV(INTn);
        }

        static void Disable()
        {
            EIMSK &= ~_BV(INTn);
        }

        static bool IsActive()
        {
            return (EIFR & _BV(INTFn)) != 0;
        }

        static void ResetFlag()
        {
            EIFR |= _BV(INTFn);
        }
    };
}